import argparse
import re

from modules.JsonSerializerCreator import JsonSerializerCreator
from modules.TOMLSerializerCreator import TomlSerializerCreator
from modules.YAMLSerializerCreator import YamlSerializerCreator

parser = argparse.ArgumentParser(description='Process converting between formats.')
parser.add_argument('-s', dest='source', type=str,
                    help='source file path')
parser.add_argument('-sf', dest='source_format', type=str,
                    help='source format')
parser.add_argument('-d', dest='destination', type=str,
                    help='destination file path')
parser.add_argument('-df', dest='format', type=str,
                    help='new format')
parser.add_argument('-cf', dest="conf_file", type=str,
                    help='configuration file')

formats = {'JSON': JsonSerializerCreator(), 'YAML': YamlSerializerCreator(), 'TOML': TomlSerializerCreator()}


def setup():
    args = parser.parse_args()
    source = args.source
    source_format = args.source_format
    destination = args.destination
    des_format = args.format
    if args.conf_file is not None:  # and args.conf_file == "config.py"
        data = {}
        try:
            with open(args.conf_file, 'r') as cf:
                for line in cf:
                    line = line.replace("\n", "")
                    (key, value) = re.findall(r"[^= \"\']+", line)
                    data[key] = value
        except IOError as e:
            print(e)

        flag = False

        if "source" in data.keys():
            source = data["source"]
        else:
            flag = True

        if "source_format" in data.keys():
            source_format = data["source_format"]
        else:
            flag = True

        if "destination" in data.keys():
            destination = data["destination"]
        else:
            flag = True

        if "format" in data.keys():
            des_format = data["format"]
        else:
            flag = True

        if not flag:
            convert(source, source_format.upper(), destination, des_format.upper())
        else:
            raise ValueError(f'Not correct config file: {formats.keys()}')

    else:
        convert(source, source_format.upper(), destination, des_format.upper())


def convert(source: str, source_format: str, destination: str, des_format):
    if des_format not in formats.keys():
        raise ValueError('Invalid format')

    try:
        with open(source, 'r') as src:
            if source_format not in formats.keys():
                raise ValueError(f'File should contain a format: {formats.keys()}')
            if source_format != des_format:
                print(f'From {source}({source_format})...')
                source_serializer = formats[source_format].create_serializer()
                serializer = formats[des_format].create_serializer()
                with open(destination, 'w') as dst:
                    data = source_serializer.loads(src.read())
                    dst.write(serializer.dumps(data))
                    print(f'To {destination}({des_format})      Completed!')
            else:
                raise ValueError(f'Source and destination format shouldn\'t be same')
    except IOError:
        print(f"Can't open file: {source}")


if __name__ == "__main__":
    setup()



from modules.Serializer import Serializer
from modules.hard_tools import ser
from modules.hard_tools import des


class JsonSerializer(Serializer):
    def dump(self, obj, file: str):
        json_str = self.dumps(obj)
        with open(file, 'w') as f:
            f.write(json_str)

    def dumps(self, obj):
        ser_obj = ser(obj)
        return ser_json(ser_obj)

    def load(self, file: str):
        try:
            with open(file, 'r') as f:
                return self.loads(f.read())
        except IOError:
            print('File IO Error')

    def loads(self, s):
        # print(des_json(s))
        return des(des_json(s))


def ser_json(obj) -> str:
    if type(obj) == dict:
        serialized = []
        for k, v in obj.items():
            serialized.append(f"{ser_json(k)}: {ser_json(v)}")
        ans = ", ".join(serialized)
        return f"{{{ans}}}"
    elif type(obj) == tuple or type(obj) == int:
        return str(obj)
    else:
        return f"\'{str(obj)}\'"


def des_json(s: str):
    output = dict()
    for i in range(len(s)):
        if i == 0 and s[i] == "{":
            if s[i + 1] == "}":
                return dict()
            return des_dict(s[1:])
        elif i == 0 and s[i] == "(":
            return find_tuple(s[1:])
        elif i == 0 and s[i] == "\'":
            return search_for_word(s[i + 1:])[0]
        else:
            return s
    return output


def des_dict(s):
    count = 1
    st = ""
    for i in range(len(s)):
        if s[i] == "{":
            count += 1
        elif s[i] == "}":
            if count == 1:
                st = s[:i]
                break
            else:
                count -= 1
    s = st
    output = dict()
    if len(s) == 0:
        return
    while 1:
        key = find_key(s)
        s = go_to_value(s)
        value = des_json(s)
        if key in TYPES:
            value = type_check(key, value)
        output[key] = value
        if isinstance(value, (str, tuple, dict, list)):
            strr = str(value)
            strr = strr.replace("\\\\", "\\")
            s = s[len(strr):]
        s = go_to_next_key(s)
        if len(s) <= 2:
            break
    return output


def go_to_value(s):
    for i in range(1, len(s)):
        if s[i] == ":":
            return s[i + 2:]


def go_to_next_key(s):
    count = 1

    for i in range(0, len(s)):  # 0 1
        if s[i] == ",":
            return s[i + 2:]

    for i in range(1, len(s)):
        if s[i] == "}":
            if count == 1:
                return s[i + 3:]
            else:
                count -= 1
        elif s[i] == "{":
            count += 1
    return ""


def find_key(s):
    if s[0] == "\'":
        for i in range(1, len(s)):
            if s[i] == "\'":
                return s[1:i]
    else:
        return des_json(s)


def find_tuple(s):
    st = ""
    output = []
    count = 1
    for i in range(len(s)):
        if s[i] == "(":
            count += 1
        elif s[i] == ")":
            if count == 1:
                st = s[:i]
                if len(st) == 0:
                    return tuple()
                elif st[0] == "\'":
                    return tuple(get_pair(st[1:len(st)]))
                else:
                    return get_dict_el(st)
            else:
                count -= 1


def get_dict_el(s):
    st = 0
    preout = []
    count = 0
    for i in range(len(s)):
        if s[i] == "{":
            count += 1
            if count == 1:
                st = i
        elif s[i] == "}":
            count -= 1
            if count == 0:
                preout.append(s[st:i + 1])
    output = []
    for i in preout:
        output.append(des_json(i))
    return tuple(output)  # tuple


TYPES = ["str", "int", "bool", "float", "None"]
COLL = ["list", "tuple", "bytes"]


def get_pair(s):
    output = []
    word, ind = search_for_word(s[0:])
    if word in TYPES:
        s = s[ind + 4: len(s) - 1]
        output.append(word)
        output.append(s)
        output[1] = type_check(output[0], output[1])
    else:
        s = s[ind + 2:]
        output.append(des_json(s))
    return output


def search_for_word(s):
    for i in range(len(s)):
        if s[i] == "\'":
            return s[0:i], i


def type_check(tp, v):
    if tp == "int":
        return int(v)
    elif tp == "float":
        return float(v)
    elif tp == "bool":
        return bool(v)
    elif tp == "None":
        return None
    elif tp == "str":
        return v
    else:
        return des_json(v)



from abc import ABC, abstractmethod


class Serializer(ABC):
    @abstractmethod
    def dump(self, obj, file):
        pass

    @abstractmethod
    def dumps(self, obj):
        pass

    @abstractmethod
    def load(self, file):
        pass

    @abstractmethod
    def loads(self, s):
        pass


from modules.Serializer import Serializer
from modules.hard_tools import ser
from modules.hard_tools import des
import yaml


class YamlSerializer(Serializer):
    def dump(self, obj, fp: str):
        ser_obj = self.dumps(obj)
        try:
            with open(fp, 'w') as f:
                f.write(ser_obj)
        except IOError:
            print("An IOError has occurred!")
        return ser_obj

    def dumps(self, obj):
        yaml_obj = change_tuple_to_list(ser(obj))
        # yaml = func_convert(yaml_obj)
        return yaml.dump(yaml_obj)

    def load(self, fp):
        try:
            with open(fp, 'r') as f:
                script = f.read()
        except IOError:
            print(f"An IOError has occurred: failed to open {fp}")
            return
        return self.loads(script)

    def loads(self, s):
        obj = dict()
        try:
            obj = yaml.safe_load(s)
            obj = change_key_from_not_tuple(obj)
        except yaml.YAMLError as exc:
            print(exc)
        return des(obj)

    '''def dumps(self, obj: object, dumper=yaml.Dumper):
        return yaml.dumps(ser(obj), Dumper=dumper)

    def dump(self, obj: object, fp: str, dumper=yaml.Dumper):
        with open(fp, "w") as file:
            file.write(self.dumps(obj, dumper=dumper))

    def loads(self, data: str, loader=yaml.FullLoader):
        return from_dict(yaml.load(data, Loader=loader))

    def load(self, fp: str, loader=yaml.FullLoader):
        with open(fp, "r") as file:
            return self.loads(file.read(), loader=loader)'''


TYPES = ["list", "tuple", "bytes", "dict",
         'co_argcount',
         'co_posonlyargcount',
         'co_kwonlyargcount',
         'co_nlocals',
         'co_stacksize',
         'co_flags',
         'co_code',
         'co_consts',
         'co_names',
         'co_varnames',
         'co_filename',
         'co_name',
         'co_firstlineno',
         'co_lnotab',
         'co_freevars',
         'co_cellvars',
         '__name__',
         '__globals__',
         '__func__',
         '__args__',
         '__builtins__',
         '__class__',
         '__object__']


def change_key_from_not_tuple(obj, flag=False):
    dc = obj.copy()
    for k, v in obj.items():
        copy = k
        if k not in TYPES and flag:
            copy = [(k, v) for k, v in ser(k).items()][0]
            dc[copy] = dc.pop(k)
        if isinstance(v, dict):
            dc[copy] = change_key_from_not_tuple(v, k == "dict")
    return dc


def change_tuple_to_list(obj):
    dt = obj.copy()
    for k, v in obj.items():
        if isinstance(v, dict):
            dt[k] = change_tuple_to_list(v)
        if k == "tuple" or k == "list" or k == "bytes":
            copy = []
            for it in list(v):
                copy.append(change_tuple_to_list(it))
            dt[k] = copy
        if isinstance(k, tuple):
            dt[k[1]] = dt.pop(k)
    return dt


'''def func_convert(obj):
    if "__func__" in obj.keys() and obj["__func__"]["__name__"] in obj["__func__"]["__globals__"].keys():
        if obj["__name__"] in obj["__globals__"]:
            flag = True'''



from modules.SerializeCreate import Creator
from modules.Serializer import Serializer
from modules.TOMLSerializer import TomlSerializer


class TomlSerializerCreator(Creator):
    def create_serializer(self) -> Serializer:
        return TomlSerializer()



import builtins
import inspect
from types import FunctionType, CodeType, LambdaType


def ser(obj):  # serialize
    return pre_ser(obj)


def pre_ser(obj):
    if isinstance(obj, (int, float, str, bool, type(None))):
        return basic_type_ser(obj)
    elif isinstance(obj, (list, tuple, bytes)):
        return tl_ser(obj)
    elif isinstance(obj, dict):
        return dict_ser(obj)
    elif inspect.iscode(obj):
        return ser_func(FunctionType(obj, {}))
    elif inspect.isfunction(obj) or inspect.ismethod(obj) or isinstance(obj, LambdaType):
        return ser_func(obj)
    elif inspect.isclass(obj):
        return ser_class(obj)
    elif inspect.isbuiltin(obj):
        return serialize_prop(obj)
    else:
        return ser_obj(obj)
        # return serialize_prop(obj)


def basic_type_ser(obj):  # str & None & bool & int & float
    temp_dict = dict()
    if isinstance(obj, str):
        temp_dict["str"] = obj
    elif isinstance(obj, type(None)):
        temp_dict["None"] = obj
    elif isinstance(obj, bool):
        temp_dict["bool"] = obj
    elif isinstance(obj, int):
        temp_dict["int"] = obj
    elif isinstance(obj, float):
        temp_dict["float"] = obj
    return temp_dict


def tl_ser(obj):  # tuple & list $ bytes
    temp_dict = dict()
    if isinstance(obj, list):
        m_key = "list"
        temp_dict["list"] = tuple(ser(el) for el in obj)
    elif isinstance(obj, tuple):
        m_key = "tuple"
        temp_dict["tuple"] = tuple(ser(el) for el in obj)
    else:
        m_key = "bytes"
        temp_dict["bytes"] = tuple(ser(el) for el in obj)
    return temp_dict


def dict_ser(obj):
    temp_dict = dict()
    temp_dict["dict"] = {}
    for k, v in obj.items():
        temp = pre_ser(k)  # bbbbbbrrrrrrrrr
        for kk, vv in temp.items():  # get just one key_type: key_value
            key = tuple((kk, vv))
            temp_dict["dict"][key] = pre_ser(v)
    return temp_dict


'''def serialize_code(obj):
    main_key = "code"
    ans = dict()
    ans[main_key] = {}

    attr = inspect.getmembers(obj)
    attr = [i for i in attr if not callable(i[1])]
    for i in attr:
        key = ser(i[0])
        val = ser(i[1])
        ans[main_key][ser(i[0])] = ser(i[1])
    ans[main_key] = tuple((k, ans[main_key][k]) for k in ans[main_key])

    return ans'''


'''def serialize_function(obj):
    FAT = [  # Function ATtr.
        "__code__",
        "__name__",
        "__defaults__",
        "__closure__",
        "__dir__",
        "__format__"
    ]
    main_key = "function"
    ans = dict()
    ans[main_key] = {}
    attr = inspect.getmembers(obj)
    attr = [i for i in attr if i[0] in FAT]
    for i in attr:
        key = ser(i[0])
        value = ser(i[1])
        ans[main_key][ser(i[0])] = ser(i[1])
        # key = pre_ser(i[0])
        # value = pre_ser(i[1])
        # if i[0] != "__closure__":
        # else:
        #    value = pre_ser(None)

        if i[0] == "__code__":
            key = ser("__globals__")
            ans[main_key][key] = {}
            names = i[1].__getattribute__("co_names")
            glob = obj.__getattribute__("__globals__")
            glob_dict = {}
            for name in names:
                if name == obj.__name__:
                    glob_dict[name] = obj.__name__
                elif name in glob and not inspect.ismodule(name) and name not in __builtins__:
                    glob_dict[name] = glob[name]
            ans[main_key][key] = ser(glob_dict)

    ans[main_key] = tuple((k, ans[main_key][k]) for k in ans[main_key])
    return ans'''


'''def nesting(level: int) -> str:
    result = ""
    for i in range(level):
        result += "\t"
    return result'''


def serialize_prop(obj):
    return {"str": f"{obj}"}


def glob_variables(func):
    gl_vars = {}
    for current_gl in func.__code__.co_names:  # search in used global variables
        if current_gl in func.__globals__:  # check if they are visible
            if inspect.isfunction(func.__globals__[current_gl]) and func.__globals__[current_gl].__name__ == func.__name__:
                gl_vars[current_gl] = str(func)  # str
            else:
                gl_vars[current_gl] = pre_ser(func.__globals__[current_gl])  # pre_ser(func.__globals__[current_gl])
    return gl_vars


# {
#   "__func__" :
#       {
#       "__name__": obj.__name,
#       "__globals__": {{:}, ...},
#       "__args__": {{:}, ...},
#       }
# }
def ser_func(obj):
    if inspect.ismethod(obj):
        obj = obj.__func__
    output = dict()
    args = dict()
    cycle = [c for c in obj.__code__.__dir__() if c.startswith('co_')]
    for c in cycle:
        attr = getattr(obj.__code__, c)
        # if isinstance(attr, bytes):
            # attr = pre_ser(attr)  # test
            # attr = attr.decode('raw_unicode_escape')
        if isinstance(attr, (list, tuple, dict)):
            converted_vals = []
            for val in attr:
                converted_vals.append(val)  # pre_ser val
            args[c] = pre_ser(converted_vals)  # pre_ser
            continue
        args[c] = pre_ser(attr)

    m_key = "__func__"
    output[m_key] = {"__name__": obj.__name__,
                     "__globals__": glob_variables(obj),
                     "__args__": args}

    return output


# {
#   "__object__" :
#       {
#       "__name__": class.__name__,
#       "__attrs__": {{:}, ...},
#       }
# }
def ser_obj(obj):
    out_attrs = dict()
    m_key = "__object__"
    cycle = [i for i in dir(obj) if not i.startswith("__")]
    for attr in cycle:
        out_attrs[attr] = pre_ser(getattr(obj, attr))
    output = {m_key: {"__name__": obj.__class__.__name__,
                      "__attrs__": out_attrs}}
    return output


# {
#   "__class__" :
#       {
#       "__name__": obj.__name__,
#       "__attrs__": {{:}, ...},
#       }
# }
def ser_class(obj):
    output = dict()
    out_attrs = dict()
    m_key = "__class__"
    attrs = [i for i in dir(obj) if not i.startswith('__') or inspect.isfunction(getattr(obj, i))]
    for attr in attrs:
        attr_value = getattr(obj, attr)
        out_attrs[attr] = pre_ser(attr_value)

    output[m_key] = {'__name__': obj.__name__,
                     '__attrs__': out_attrs}
    return output


SIMPLE_TYPES = ["str", "int", "float", "bool", "None"]
L_T_B_TYPES = ["list", "tuple", "bytes"]


def des(obj):
    for k, v in obj.items():
        if k in SIMPLE_TYPES:
            return des_types(v)
        elif k in L_T_B_TYPES:
            return des_ltb(k, v)
        elif k == "dict":
            return des_dict(v)
        elif k == "__func__":
            return des_func(v)
        elif k == "__class__":
            return des_class(v)
        elif k == "__object__":
            return des_obj(v)


def des_types(obj):
    return obj


def des_ltb(tp, obj):
    output = []
    for el in obj:
        output.append(des(el))

    if tp == "tuple":
        output = tuple(output)
    elif tp == "bytes":
        output = bytes(output)
    return output


def des_dict(obj):
    output = dict()
    for k, v in obj.items():
        key = des({k[0]: k[1]})
        output[key] = des(v)

    return output


def des_func(obj):
    flag = False
    args = obj["__args__"]
    if "__globals__" not in obj.keys():
        obj["__globals__"] = {}
    globs = obj["__globals__"]
    if obj["__name__"] in globs.keys():
        flag = True
    globs["__builtins__"] = builtins
    for key in obj["__globals__"]:
        if len(args["co_names"]) != 0 and key in des(args["co_names"]):  # list
            if key != des(args["co_name"]):
                globs[key] = des(obj["__globals__"][key])

    for k, v in args.items():
        if k == "co_consts":
            consts = []
            for val in des(args["co_consts"]):
                if inspect.isfunction(val) or inspect.ismethod(val) or isinstance(val, LambdaType):
                    # val = des(val)
                    consts.append(val.__code__)
                    continue
                consts.append(val)
            args[k] = consts
        else:
            args[k] = des(args[k])

    code = CodeType(args['co_argcount'],
                    args['co_posonlyargcount'],
                    args['co_kwonlyargcount'],
                    args['co_nlocals'],
                    args['co_stacksize'],
                    args['co_flags'],
                    bytes(args['co_code']),  # , 'raw_unicode_escape'
                    tuple(args['co_consts']),
                    tuple(args['co_names']),
                    tuple(args['co_varnames']),
                    args['co_filename'],
                    args['co_name'],
                    args['co_firstlineno'],
                    bytes(args['co_lnotab']),  # , 'raw_unicode_escape'
                    tuple(args['co_freevars']),
                    tuple(args['co_cellvars']))
    output = FunctionType(code, globs)
    # print(output.__globals__)
    if flag:
        output.__globals__[obj["__name__"]] = output
    # print(output.__globals__)
    return output


def des_obj(obj):
    '''dct = {}
    output = type(obj["__name__"], (), {})
    attrs = obj["__attrs__"]
    for i in attrs:
        # dct[i] = des(attrs[i])
        setattr(output, i, des(attrs[i]))
    return output'''
    # return type(obj["__name__"], (), dct)
    dct = {}
    # res = type(obj["__name__"], (), {})
    attrs = obj["__attrs__"]
    func = []
    for i in attrs:
        ob = des(attrs[i])
        if inspect.isfunction(ob) and "self" in ob.__code__.co_varnames:
            func.append(ob)
        else:
            dct[i] = ob
    output = type(obj["__name__"], (), dct)
    for i in func:
        setattr(output, i.__name__, i.__get__(output))
        # name = i.__name__
        # output.name = i.__get__(output)
    return output
    # return type(obj["__name__"], (), dct)


def des_class(obj):
    dct = {}
    attrs = obj["__attrs__"]
    # for i in attrs:
    #    dct[i] = des(attrs[i])

    func = []
    for i in attrs:
        ob = des(attrs[i])
        if inspect.isfunction(ob) and "self" in ob.__code__.co_varnames:
            func.append(ob)
        else:
            dct[i] = ob
    output = type(obj["__name__"], (), dct)
    for i in func:
        setattr(output, i.__name__, i.__get__(output))
        # name = i.__name__
        # output.name = i.__get__(output)
    return output

    # for attr, val in obj.items():
    #    if attr != "__name__":
    #        dct[attr] = des(val)
    # return type(obj["__name__"], (), dct)


'''def serialize_instance(instance_obj):
    ans = dict()
    type = re.search(r"\'(\w+)\'", str(type(instance_obj))).group(1)
    ans[type] = {}
    members = inspect.getmembers(instance_obj)
    members = [i for i in members if not callable(i[1])]
    for i in members:
        key = ser(i[0])
        val = ser(i[1])
        ans[type][key] = val
    ans[type] = tuple((k, ans[type][k]) for k in ans[type])

    return ans'''




from modules.SerializeCreate import Creator
from modules.Serializer import Serializer
from modules.YAMLSerializer import YamlSerializer


class YamlSerializerCreator(Creator):
    def create_serializer(self) -> Serializer:
        return YamlSerializer()


from modules.SerializeCreate import Creator
from modules.Serializer import Serializer
from modules.JsonSerializer import JsonSerializer


class JsonSerializerCreator(Creator):
    def create_serializer(self) -> Serializer:
        return JsonSerializer()



import re
import sys
from toml.decoder import InlineTableDict
from modules.Serializer import Serializer
from modules.hard_tools import ser
from modules.hard_tools import des
import toml


class TomlSerializer(Serializer):
    def __init__(self):
        toml.TomlEncoder.dump_sections = my_dump_sections

    def dump(self, obj, file: str):
        output = self.dumps(obj)
        with open(file, 'w') as f:
            f.write(output)

    def dumps(self, obj):
        toml_obj = change_tuple_to_list(ser(obj))
        return toml.dumps(toml_obj)

    def load(self, file):
        try:
            with open(file, 'r') as file:
                return self.loads(file.read())
        except IOError:
            print('File IO Error')

    def loads(self, s):
        s = toml.loads(s)
        s = from_toml_obj(s)
        # print(s)
        return des(s)


def from_toml_obj(dc):
    dic = dc.copy()
    for k, v in dc.items():
        key = k
        if isinstance(k, str) and k[0] == "(":
            key = str_to_tuple(k)
            dic[key] = dic.pop(k)
        if isinstance(v, dict):  # and len(v) != 1
            dic[key] = from_toml_obj(dic.pop(key))
        if isinstance(v, list):
            dic[key] = from_toml_list(dic.pop(key))
    return dic


def str_to_tuple(s):
    output = list()
    ls = s.split(", ")
    for it in ls:
        el = it
        el = el.replace('(', "")
        el = el.replace(')', "")
        el = el.replace("'", "")
        output.append(el)
    return tuple(output)


def change_tuple_to_list(obj):
    for k, v in obj.items():
        if isinstance(v, dict):
            obj[k] = change_tuple_to_list(v)
        if k == "tuple" or k == "list" or k == "bytes":
            copy_list = list()
            for it in list(v):
                copy_it = it
                if isinstance(it, dict):
                    copy_it = change_tuple_to_list(it)
                copy_list.append(copy_it)
            obj[k] = copy_list
    return obj


def from_toml_list(obj):
    output = list()
    for it in obj:
        copy = it
        if isinstance(it, dict):
            copy = from_toml_obj(it)
        output.append(copy)
    return output


def my_dump_sections(self, o, sup):
    unicode = str
    retstr = ""
    if sup != "" and sup[-1] != ".":
        sup += '.'
    retdict = self._dict()
    arraystr = ""
    for section in o:
        string_repres = unicode(section)  ## MyEdit
        qsection = section
        if not re.match(r'^[A-Za-z0-9_-]+$', string_repres):
            qsection = dump_str(string_repres)
        if not isinstance(o[section], dict):
            arrayoftables = False
            if isinstance(o[section], list):
                for a in o[section]:
                    if isinstance(a, dict):
                        arrayoftables = True
            if arrayoftables:
                for a in o[section]:
                    arraytabstr = "\n"
                    arraystr += "[[" + sup + qsection + "]]\n"
                    s, d = self.dump_sections(a, sup + qsection)
                    if s:
                        if s[0] == "[":
                            arraytabstr += s
                        else:
                            arraystr += s
                    while d:
                        newd = self._dict()
                        for dsec in d:
                            s1, d1 = self.dump_sections(d[dsec], sup +
                                                        qsection + "." +
                                                        dsec)
                            if s1:
                                arraytabstr += ("[" + sup + qsection +
                                                "." + dsec + "]\n")
                                arraytabstr += s1
                            for s1 in d1:
                                newd[dsec + "." + s1] = d1[s1]
                        d = newd
                    arraystr += arraytabstr
            else:
                if o[section] is not None:
                    retstr += (qsection + " = " +
                               unicode(self.dump_value(o[section])) + '\n')
        elif self.preserve and isinstance(o[section], InlineTableDict):
            retstr += (qsection + " = " +
                       self.dump_inline_table(o[section]))
        else:
            retdict[qsection] = o[section]
    retstr += arraystr
    return (retstr, retdict)


def dump_str(v):
    unicode = str
    if sys.version_info < (3,) and hasattr(v, 'decode') and isinstance(v, str):
        v = v.decode('utf-8')
    v = "%r" % v
    if v[0] == 'u':
        v = v[1:]
    singlequote = v.startswith("'")
    if singlequote or v.startswith('"'):
        v = v[1:-1]
    if singlequote:
        v = v.replace("\\'", "'")
        v = v.replace('"', '\\"')
    v = v.split("\\x")
    while len(v) > 1:
        i = -1
        if not v[0]:
            v = v[1:]
        v[0] = v[0].replace("\\\\", "\\")
        # No, I don't know why != works and == breaks
        joinx = v[0][i] != "\\"
        while v[0][:i] and v[0][i] == "\\":
            joinx = not joinx
            i -= 1
        if joinx:
            joiner = "x"
        else:
            joiner = "u00"
        v = [v[0] + joiner + v[1]] + v[2:]
    return unicode('"' + v[0] + '"')




from setuptools import setup

setup(
    name="hardser",
    version="0.0.1.7",
    author="Mikhail Hancharou",
    author_email="engeneer2003@mail.ru",
    packages=['modules', 'Converter'],
    entry_points={
        'console_scripts': ['hardser=Converter.Converter:setup'],
    },
    install_requires=[
        'coverage==6.3.2',
        'entrypoints==0.3',
        'pytest==7.1.2',
        'pytest-cov==3.0.0',
        'PyYAML==5.3.1',
        'simplejson==3.16.0',
        'toml==0.10.2',
        'virtualenv==20.0.17',
    ]
)



from modules.JsonSerializerCreator import JsonSerializerCreator
from modules.TOMLSerializerCreator import TomlSerializerCreator
from modules.YAMLSerializerCreator import YamlSerializerCreator


class ParserFactory:
    @staticmethod
    def create_parser(name: str):
        name = name.lower()
        if name == "json":
            # parser = ()
            return JsonSerializerCreator().create_serializer()

        if name == "yaml":
            return YamlSerializerCreator().create_serializer()

        if name == "toml":
            return TomlSerializerCreator().create_serializer()

        raise ValueError(f"Format {name} is not supported")
